/*
    Copyright (C) 2006 Carlos Olmedo Escobar <arroba2puntos@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "cpu.h"

int main()	// Just for testing
{
    Processor cpu;
    int i;
    QMap<QString, float> map_of_percetages;
        
    for ( i = 0; i < 5; i++ )
        {
        cpu.getUsage( &map_of_percetages );
        std::cout << "idle: " << map_of_percetages.value( "idle" ) << " %\n";
        sleep( 1 );
        }

    for ( i = 0; i < 5; i++ )
        {
      	cpu.getUsage( &map_of_percetages, 0 );
      	std::cout << "user: " << map_of_percetages.value( "user" ) << " %\n";
        sleep( 1 );
        }

}



Processor::Processor()
{
    file_proc_stat.setFileName("/proc/stat");
}



Processor::~Processor()
{
    if ( file_proc_stat.isOpen() )
        file_proc_stat.close();
}



bool Processor::getUsage( QMap<QString, float> * map_to_fill )
{
    if ( map_to_fill == NULL )
        return false;
    
    return __getUsage( map_to_fill, -1 );
}



bool Processor::getUsage( QMap<QString, float> * map_to_fill, short cpu_number )
{
    if ( map_to_fill == NULL )
        return false;
    
    if ( cpu_number < 0 )
        return false;
    else
        return __getUsage( map_to_fill, cpu_number );
}



bool Processor::__getUsage( QMap<QString, float> * map_to_fill, short cpu_number )
{
    register short current_line;

// Buffering this file is nonsense because its already in memory and its data is valid only once
    if ( file_proc_stat.isOpen() )
        file_proc_stat.seek( 0 );		// Reopen is not necessary
    else
        if ( file_proc_stat.open( QIODevice::ReadOnly | QIODevice::Text | QIODevice::Unbuffered ) == false )
            return false;
    
    for ( current_line = 0; current_line <= cpu_number + 1; current_line++ )
        {
        if ( file_proc_stat.readLine( buffer, MAX_SIZE_OF_LINE - 1 ) == -1 )
            return false;
        }

    if ( parseLine() == false )
        return false;
    
    map_to_fill->insert( "user", (float) (real_cpu_times[CPU_USER] * 100.0) / (float) total_time );
    map_to_fill->insert( "nice", (float) (real_cpu_times[CPU_SYSTEM] * 100.0) / (float) total_time );
    map_to_fill->insert( "system", (float) (real_cpu_times[CPU_NICE] * 100.0) / (float) total_time );
    map_to_fill->insert( "idle", (float) (real_cpu_times[CPU_IDLE] * 100.0) / (float) total_time );
    map_to_fill->insert( "iowait", (float) (real_cpu_times[CPU_IOWAIT] * 100.0) / (float) total_time );
    map_to_fill->insert( "hw_irq", (float) (real_cpu_times[CPU_HW_IRQ] * 100.0) / (float) total_time );
    map_to_fill->insert( "sw_interr", (float) (real_cpu_times[CPU_SW_INTERR] * 100.0) / (float) total_time );
    
    return true;
}



bool Processor::parseLine()
{
	register short i;
    char cpu_name[10];
    static unsigned long long saved_cpu_times[CPU_STATES];
    static unsigned long long current_cpu_times[CPU_STATES];
    
    if ( sscanf( buffer, "%s %Lu %Lu %Lu %Lu %Lu %Lu %Lu", cpu_name, &current_cpu_times[0], 
            &current_cpu_times[1], &current_cpu_times[2], &current_cpu_times[3], &current_cpu_times[4], 
            &current_cpu_times[5], &current_cpu_times[6] ) < 7 )
        return false;
    
    total_time = 0;
    for ( i = 0; i < CPU_STATES; i++ )
        {
        real_cpu_times[i] = current_cpu_times[i] - saved_cpu_times[i];
        total_time += real_cpu_times[i];
        saved_cpu_times[i] = current_cpu_times[i];
        }

    return true;
}
