/*
    Copyright (C) 2006 Carlos Olmedo Escobar <arroba2puntos@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef CPU_H
#define CPU_H

#include <QFile>
#include <QMap>
#include <QString>

#include <stdio.h>
#include <iostream> 

#define MAX_SIZE_OF_LINE 100
#define CPU_STATES 7
#define CPU_USER 0
#define CPU_SYSTEM 1
#define CPU_NICE 2
#define CPU_IDLE 3
#define CPU_IOWAIT 4
#define CPU_HW_IRQ 5
#define CPU_SW_INTERR 6


/**
 * This class retrieves information about the cpu usage in Linux.
 *
 * @author Carlos Olmedo Escobar <arroba2puntos@gmail.com>
 */
class Processor
{

public:
    /**
     * Constructs a processor object.
     */
    Processor();
    
    /**
     * Destroys a processor object.
     */
    ~Processor();
    
    /**
     * Retrieves the diferent usage percentages from the overall cpus
     * @param map_to_fill The associative container where the data is 
     * stored. Key values refers to types of percentages.
     * Key values: "user", "nice", "system" and "idle".
     * Non transportable values: "iowait", "hw_irq" and "sw_interr".
     * @return false if occurs an error, true otherwise.
     */
    bool getUsage( QMap<QString, float> * map_to_fill );

    /**
     * Retrieves the diferent usage percentages from each cpu. This
     * method is not transportable.
     * @param map_to_fill The associative container where the data is
     * stored. Key values refers to types of percentages.
     * Key values: "user", "nice", "system" and "idle".
     * Non transportable values: "iowait", "hw_irq" and "sw_interr".
     * @param cpu_number Index of cpu to retrieve the info (0..n).
     * @return false if occurs an error, true otherwise.
     */
    bool getUsage( QMap<QString, float> * map_to_fill, short cpu_number );

private:
    QFile file_proc_stat;
    unsigned long long real_cpu_times[CPU_STATES];
    unsigned long long total_time;
    char buffer[MAX_SIZE_OF_LINE];

    /**
     * An internal method that really does the job.
     * @param map_to_fill The associative container where the data is
     * stored. Key values refers to types of percentages.
     * Key values: "user", "nice", "system" and "idle".
     * Non transportable values: "iowait", "hw_irq" and "sw_interr".
     * @param cpu_number Index of cpu to retrieve the info (0..n).
     * @return false if occurs an error, true otherwise.
     */
    bool __getUsage( QMap<QString, float> * map_to_fill, short cpu_number );

    /**
     * Parses a line of /proc/stat and calculates the percentages
     * @return false if occurs an error, true otherwise.
     */
    bool parseLine();

};

#endif
